@@ -29,6 +29,7 @@
 #include "nm_manager_proxy.h"
 #include "nm_settings_proxy.h"
 #include "nm_settings_connection_proxy.h"
+#include "networkproperties.h"
 
 #define NM_SERVICE "org.freedesktop.NetworkManager"
 #define NM_PATH "/org/freedesktop/NetworkManager"
@@ -36,6 +37,7 @@
 #define NM_DEVICE_IFACE "org.freedesktop.NetworkManager.Device"
 #define NM_DEVICE_WIRELESS_IFACE "org.freedesktop.NetworkManager.Device.Wireless"
 #define NM_ACTIVE_CONNECTION_IFACE "org.freedesktop.NetworkManager.Connection.Active"
+#define NM_IP4_CONFIG_IFACE "org.freedesktop.NetworkManager.IP4Config"
 
 typedef QMap<QString,QVariantMap> ConfigurationData;
 Q_DECLARE_METATYPE(ConfigurationData)
@@ -48,6 +50,7 @@ WifiDbusHelper::WifiDbusHelper(QObject *parent)
 WifiDbusHelper::WifiDbusHelper(const QDBusConnection &dbus, QObject *parent)
     : QObject(parent)
     , m_systemBusConnection(dbus)
+    , m_networkPropertiesModel(this)
 {
     qDBusRegisterMetaType<ConfigurationData>();
 }
@@ -295,39 +298,9 @@ void WifiDbusHelper::nmDeviceStateChanged(uint newState,
     Q_EMIT (deviceStateChanged(newState, reason));
 }
 
-QString WifiDbusHelper::getWifiIpAddress()
-{
-    OrgFreedesktopNetworkManagerInterface mgr(NM_SERVICE,
-                                              NM_PATH,
-                                              m_systemBusConnection);
-
-    // find the first wlan adapter for now
-    auto reply1 = mgr.GetDevices();
-    reply1.waitForFinished();
-    if(!reply1.isValid()) {
-        qWarning() << "Could not get network device: " << reply1.error().message() << "\n";
-        return QString();
-    }
-    auto devices = reply1.value();
-
-    for (const auto &d : devices) {
-        QDBusInterface iface(NM_SERVICE, d.path(), NM_DEVICE_IFACE, m_systemBusConnection);
-        auto type_v = iface.property("DeviceType");
-        if (type_v.toUInt() == 2 /* NM_DEVICE_TYPE_WIFI */) {
-            auto ip4name = iface.property("IpInterface").toString();
-
-            QList<QNetworkAddressEntry> entries =
-                QNetworkInterface::interfaceFromName(ip4name).addressEntries();
-            if (entries.size() > 0) {
-                return entries[0].ip().toString();
-            }
-            break;
-        }
-    }
-
-    return QString();
-}
-
+/**
+ * taking care of getting saved password from given connection
+ */
 struct Network : public QObject
 {
     struct DontCare : public std::exception {};
@@ -424,7 +397,7 @@ struct Network : public QObject
         if (timestamp != 0) {
 
             QString secretsType;
-            if (keymgmt == "wpa-psk" && authalg == "open") {
+            if ((keymgmt == "wpa-psk" && authalg == "open") || keymgmt == "sae") {
                 secretsType = "802-11-wireless-security";
             } else if (keymgmt == "wpa-eap" || keymgmt == "ieee8021x") {
                 secretsType = "802-1x";
@@ -443,7 +416,7 @@ struct Network : public QObject
 
                 if (keymgmt == "none") {
                     password = secrects_security["wep-key0"].toString();
-                } else if (keymgmt == "wpa-psk" && authalg == "open") {
+                } else if ((keymgmt == "wpa-psk" && authalg == "open") || keymgmt == "sae") {
                     password = secrects_security["psk"].toString();
                 } else if (keymgmt == "wpa-eap" || keymgmt == "ieee8021x") {
                     password = secrects_security["password"].toString();
@@ -489,6 +462,73 @@ struct Network : public QObject
     QMap<QString, QVariantMap> settings;
 };
 
+NetworkPropertiesModel * WifiDbusHelper::networkPropertiesModel()
+{
+    OrgFreedesktopNetworkManagerInterface mgr(NM_SERVICE,
+                                              NM_PATH,
+                                              m_systemBusConnection);
+
+    // find the first wlan adapter for now
+    auto reply1 = mgr.GetDevices();
+    reply1.waitForFinished();
+    if(!reply1.isValid()) {
+        qWarning() << "Could not get network device: " << reply1.error().message() << "\n";
+        return &m_networkPropertiesModel;
+    }
+    auto devices = reply1.value();
+
+    QList<QString> currentIpAddresses = QStringList();
+
+    for (const auto &d : devices) {
+        QDBusInterface iface(NM_SERVICE, d.path(), NM_DEVICE_IFACE, m_systemBusConnection);
+        auto type_v = iface.property("DeviceType");
+        if (type_v.toUInt() == 2 /* NM_DEVICE_TYPE_WIFI */) {
+            auto ac_path_var = iface.property("ActiveConnection");
+            QString ac_path = ac_path_var.value<QDBusObjectPath>().path();
+            QDBusInterface ac_iface(NM_SERVICE, ac_path, NM_ACTIVE_CONNECTION_IFACE, m_systemBusConnection);
+
+            // we take connection in order to have path for network.device for password
+            auto conn_path_var = ac_iface.property("Connection");
+
+            QVariant ipv4_var = ac_iface.property("Ip4Config");
+            if (ipv4_var.isValid() && ipv4_var.canConvert<QDBusObjectPath>()) {
+                QDBusObjectPath ipv4_path = ipv4_var.value<QDBusObjectPath>();
+                QDBusInterface ipv4_iface(NM_SERVICE, ipv4_path.path(), NM_IP4_CONFIG_IFACE, m_systemBusConnection);
+
+                QVariant gatewayVar = ipv4_iface.property("Gateway");
+                if (gatewayVar.isValid()) {
+                    QString gateway = gatewayVar.toString();
+                    m_networkPropertiesModel.setGateway(gateway);
+                } else {
+                    qDebug() << "Gateway is not valid";
+                }
+
+            } else {
+                qDebug() << "Ip4Config is not valid or cannot be converted to QDBusObjectPath";
+            }
+
+            Network network(conn_path_var.value<QDBusObjectPath>().path());
+            m_networkPropertiesModel.setPassword(network.password);
+            auto ip4name = iface.property("IpInterface").toString();
+
+            // https://doc.qt.io/qt-5/qnetworkaddressentry.html
+            QList<QNetworkAddressEntry> entries = QNetworkInterface::interfaceFromName(ip4name).addressEntries();
+            if (entries.size() > 0) {
+                for(int idx = 0 ; idx < entries.size() ; ++idx) {
+                    QString ipv6Address = entries[idx].ip().toString();
+                    int percentIndex = ipv6Address.indexOf('%');
+                    currentIpAddresses.append((percentIndex != -1) ? ipv6Address.left(percentIndex) : ipv6Address);
+                }
+                m_networkPropertiesModel.setNetmask(entries[0].netmask().toString());
+                m_networkPropertiesModel.setIpAddress(currentIpAddresses);
+            }
+            break;
+        }
+    }
+    return &m_networkPropertiesModel;
+}
+
+
 QList<QStringList> WifiDbusHelper::getPreviouslyConnectedWifiNetworks() {
     QList<QStringList> networks;
 
